package com.icomsys;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.icomsys.dsl.MessageSendServiceDsl;
import com.icomsys.util.CommonBeanUtils;
import com.icomsys.vo.*;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

public class MessagingConnector {

    private final String sms;
    private final String lms;
    private final String mms;
    private final String kakao;
    private final String apiKey;
    public MessageSendServiceDsl messageSendServiceDsl;
    private final HttpClient httpClient;

    public MessagingConnector(String sms, String lms, String mms, String kakao, String apiKey) {
        this.sms = sms;
        this.lms = lms;
        this.mms = mms;
        this.kakao = kakao;
        this.apiKey = apiKey;
        this.httpClient = HttpClient.newHttpClient();
    }

    public MessageSendResponse messageDiv(MessageSendRequest messageSendRequest) throws IOException, InterruptedException {
        String url = sms;
        String key = UUID.randomUUID().toString().substring(0, 6);
        LocalDate now = LocalDate.now();
        messageSendRequest.setUserKey(now.toString().substring(2).replace("-", "") + key);

        if (messageSendRequest.getSendType() == null || messageSendRequest.getSendType().isEmpty()) {
            if (messageSendRequest.getContents().getBytes(StandardCharsets.UTF_8).length < 91) {
                messageSendRequest.setSendType("SMS");
                return this.messageSend(messageSendRequest, url);
            } else {
                messageSendRequest.setSendType("LMS");
                url = lms;
                return this.messageSend(messageSendRequest, url);
            }
        } else {
            switch (messageSendRequest.getSendType()) {
                case "SMS":
                    return this.messageSend(messageSendRequest, url);
                case "LMS":
                    url = lms;
                    return this.messageSend(messageSendRequest, url);
                case "MMS":
                    url = mms;
                    return this.messageSend(messageSendRequest, url);
                case "KAKAO":
                    url = kakao;
                    return this.messageSend(messageSendRequest, url);
                default:
                    return null;
            }
        }
    }

    public MessageSendResponse messageSend(MessageSendRequest messageSendRequest, String url) throws IOException, InterruptedException {
        MessageSendResponse messageSendResponse = new MessageSendResponse();
        SejongMessageSendResponse sejongMessageSendResponse = null;

        try {
            ObjectMapper objectMapper = new ObjectMapper();
            Map<String, String> body = prepareRequestBody(messageSendRequest, objectMapper);
            System.out.println("body : " + body);

            String jsonBody = objectMapper.writeValueAsString(body);
            System.out.println("jsonbody : " + jsonBody);

            HttpRequest request = HttpRequest.newBuilder()
                    .uri(URI.create(url))
                    .header("Content-Type", "application/json")
                    .header("sejongApiKey", apiKey)
                    .POST(HttpRequest.BodyPublishers.ofString(jsonBody))
                    .build();

            HttpResponse<String> response = httpClient.send(request, HttpResponse.BodyHandlers.ofString());
            System.out.println("responseCode : " + response.statusCode());
            System.out.println("responseBody : " + response.body());
            System.out.println("responseFull : " + response);

            if (response.statusCode() == 200) {
                sejongMessageSendResponse = objectMapper.readValue(response.body(), SejongMessageSendResponse.class);
            } else {
                messageSendResponse.setCode("500");
                messageSendResponse.setMessage("서버 통신 오류");
            }
        } catch (Exception e) {
            messageSendResponse.setCode("500");
            messageSendResponse.setMessage("서버 통신 오류");
        }

        messageSendResponse = this.responseChange(sejongMessageSendResponse, messageSendRequest);
        messageSendServiceDsl.insertSendLog(sejongMessageSendResponse, messageSendRequest, messageSendResponse);

        if ("KAKAO".equals(messageSendRequest.getSendType()) && "Y".equals(messageSendRequest.getOptionYn())) {
            if (!Objects.equals(sejongMessageSendResponse.getCode(), "200")) {
                messageSendRequest.setSendType("");
                messageSendResponse = this.messageDiv(messageSendRequest);
            }
        }

        return messageSendResponse;
    }

    private Map<String, String> prepareRequestBody(MessageSendRequest messageSendRequest, ObjectMapper objectMapper) {
        switch (messageSendRequest.getSendType()) {
            case "SMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongSmsSendRequest.class), new TypeReference<>() {});
            case "LMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongLmsSendRequest.class), new TypeReference<>() {});
            case "MMS":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongMmsSendRequest.class), new TypeReference<>() {});
            case "KAKAO":
                return objectMapper.convertValue(CommonBeanUtils.convertType(messageSendRequest, SejongKakaoSendRequest.class), new TypeReference<>() {});
            default:
                throw new IllegalArgumentException("Unsupported send type");
        }
    }

    public MessageSendResponse responseChange(SejongMessageSendResponse response, MessageSendRequest request) {
        MessageSendResponse messageSendResponse = new MessageSendResponse();
        messageSendResponse.setSendType(request.getSendType());
        messageSendResponse.setSendCode(response != null ? response.getSendCode() : null);

        if (response == null || response.getCode() == null) {
            messageSendResponse.setCode("500");
            messageSendResponse.setMessage("서버 오류");
            return messageSendResponse;
        }

        int code = Integer.parseInt(response.getCode());
        if (code == 200) {
            messageSendResponse.setCode("200");
            messageSendResponse.setMessage("메세지 전송 요청 성공");
        } else if (code >= 300 && code < 400) {
            messageSendResponse.setCode("300");
            messageSendResponse.setMessage("Parameter 확인");
        } else if (code >= 400 && code < 500) {
            messageSendResponse.setCode("400");
            messageSendResponse.setMessage("서버 통신 실패");
        } else if (code >= 500) {
            messageSendResponse.setCode("600");
            messageSendResponse.setMessage("메세지 전송 오류");
        }

        return messageSendResponse;
    }
}